"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AsyncWalkBuilder = exports.WalkBuilder = void 0;
const walk_1 = require("./walk");
class CallbacksBuilder {
    constructor(cbs, source) {
        this.cbs = cbs;
        this.source = source;
        this.callback = { callback: () => { } };
    }
    withExecutionOrder(order) {
        this.callback.executionOrder = order;
        return this;
    }
    filteredByNodeTypes(...types) {
        this.callback.nodeTypeFilters = types;
        return this;
    }
    filteredByKeys(...keys) {
        this.callback.keyFilters = keys;
        return this;
    }
    filteredByPositions(...positions) {
        this.callback.positionFilter = positions;
        return this;
    }
    done() {
        return this.source.withCallbacks(...this.cbs.map(cb => (Object.assign(Object.assign({}, this.callback), { callback: cb }))));
    }
}
class BaseWalkBuilder {
    constructor() {
        this.config = {};
    }
    withConfig(config) {
        this.config = config;
        return this;
    }
    withTraversalMode(traversalMode) {
        this.config.traversalMode = traversalMode;
        return this;
    }
    withGraphMode(graphMode) {
        this.config.graphMode = graphMode;
        return this;
    }
    withRootObjectCallbacks(val) {
        this.config.rootObjectCallbacks = val;
        return this;
    }
    withRunningCallbacks(val) {
        this.config.runCallbacks = val;
        return this;
    }
    withConfiguredCallbacks(...callbacks) {
        return new CallbacksBuilder(callbacks, this);
    }
    withConfiguredCallback(callback) {
        return this.withConfiguredCallbacks(callback);
    }
    withCallback(callback) {
        return this.withCallbacks(callback);
    }
    withCallbacks(...callbacks) {
        if (!this.config.callbacks)
            this.config.callbacks = [];
        this.config.callbacks.push(...callbacks);
        return this;
    }
}
class WalkBuilder extends BaseWalkBuilder {
    walk(obj) {
        return walk_1.walk(obj, this.config);
    }
    withSimpleCallback(callback) {
        return this.withSimpleCallbacks(callback);
    }
    withSimpleCallbacks(...callbacks) {
        return this.withCallbacks(...(callbacks.map(c => ({ callback: c }))));
    }
}
exports.WalkBuilder = WalkBuilder;
class AsyncWalkBuilder extends BaseWalkBuilder {
    withParallelizeAsyncCallbacks(val) {
        this.config.parallelizeAsyncCallbacks = val;
        return this;
    }
    withSimpleCallback(callback) {
        return this.withSimpleCallbacks(callback);
    }
    withSimpleCallbacks(...callbacks) {
        return this.withCallbacks(...(callbacks.map(c => ({ callback: c }))));
    }
    walk(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            return walk_1.walk(obj, this.config);
        });
    }
}
exports.AsyncWalkBuilder = AsyncWalkBuilder;
const logCallback = (node) => console.log(node);
const myObject = {};
new WalkBuilder()
    .withSimpleCallback(logCallback)
    .withCallback({
    keyFilters: ['myKey'],
    positionFilters: ['postWalk'],
    nodeTypeFilters: ['object'],
    executionOrder: 0,
    callback: logCallback
})
    .withGraphMode('graph')
    .withTraversalMode('breadth')
    .withRunningCallbacks(true)
    .withRootObjectCallbacks(true)
    .withConfiguredCallback(logCallback)
    .filteredByKeys('key1', 'key2')
    .filteredByNodeTypes('object', 'array')
    .filteredByPositions('postWalk', 'preWalk')
    .withExecutionOrder(1)
    .done()
    .walk(myObject);
