"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CallbackStacker = exports.updateObjectViaPathString = exports.unique = exports.executionOrderSort = void 0;
function executionOrderSort(a, b) {
    const _a = a.executionOrder || 0;
    const _b = b.executionOrder || 0;
    return _a < _b ? -1 : _a > _b ? 1 : 0;
}
exports.executionOrderSort = executionOrderSort;
function unique(arr) {
    const seen = new Set();
    return arr.filter((val) => {
        if (seen.has(val))
            return;
        seen.add(val);
        return val;
    });
}
exports.unique = unique;
function updateObjectViaPathString(obj, val, path, delimiter) {
    const block = path.split(delimiter).slice(1);
    while (block.length > 1)
        obj = obj[block.shift()];
    obj[block.shift()] = val;
}
exports.updateObjectViaPathString = updateObjectViaPathString;
class CallbackStacker {
    constructor(executor) {
        this.executor = executor;
        this.lookup = {};
    }
    push(key, node, callbacks) {
        this.lookup[key] = {
            trigger: node.id,
            fn: () => this.executor(callbacks, node)
        };
    }
    executeOne(node, callbacks) {
        return this.executor(callbacks, node);
    }
    *execute(nodeId) {
        let next = this.lookup[nodeId];
        delete this.lookup[nodeId];
        while (next) {
            yield next.fn();
            const trigger = next.trigger;
            next = this.lookup[trigger];
            delete this.lookup[trigger];
        }
    }
}
exports.CallbackStacker = CallbackStacker;
